%Script converts a g-file with C-records (and large correlation-st dev
%matrices) to a *.gps file for input in Starnet

clear all
close all
clc;

%specify the file to convert
INPUT = {'network-final.gfile'}; %specify name of g-file
OUTPUT = {'mkea03_gnss.dat'}; %specify name of output *.gps file
SERFIL = {'serfil'}; %specify name of serfil or file to convert SSNs to desired mark identifiers

%%
str = char(INPUT);
fid = fopen(str);
C = textscan(fid, '%s','Delimiter','');
fclose(fid);
C = C{:};
%%
%Find all sessions in the g-file, C-records, F-records, and D matrices

iB = find(strncmp(C,'B',1)==1); %index to all B-records in the g-file
iC = find(strncmp(C,'C',1)==1); %index to all C-records in the g-file
iF = find(strncmp(C,'F',1)==1); %index to all F-records in the g-file
iE = find(strncmp(C,'D',1)==1); %index to all D matrices in the g-file

iCF = union(iC,iF); %combined index to all vectors (C and F) in g-file
%%
%Assign start dates etc. from the B-record for each session
for i = 1:length(iB)
    bdata=[];
    bdata = char(C(iB(i)));
    session(i).year = str2double(bdata(2:5));
    session(i).month = str2double(bdata(6:7));
    session(i).day = str2double(bdata(8:9));
    session(i).hr = str2double(bdata(10:11));
    session(i).min = str2double(bdata(12:13));
    session(i).software = cellstr(bdata(28:42));
end

%%
%Assign all vector component records for each individual session
for i = 1:length(iB)
    fstart = find(iCF > iB(i));
    if i < length(iB)
        fend = find(iCF < iB(i+1));
    else
        fend = fstart;
    end
    f = intersect(fstart,fend); %index to all vector records in a session
    
    
    for k = 1:length(f)
        if strncmp(C(iCF(k)),'C',1) == 1
            fdata = [];
            fdata = char(C(iCF(f(k))));
            session(i).fromsta(k,1) = str2num(fdata(2:5)); %all from stations in a session (by SSN)
            session(i).tosta(k,1) = str2num(fdata(6:9)); %all to stations in a session        
               
            session(i).dx(k,1) = str2num(fdata(10:20))/1e4; %delta X in meters
            session(i).dy(k,1) = str2num(fdata(26:36))/1e4; %delta Y in meters
            session(i).dz(k,1) = str2num(fdata(42:52))/1e4; %delta Z in meters
            session(i).stdx(k,1) = str2num(fdata(21:25))/1e4; %std X in meters
            session(i).stdy(k,1) = str2num(fdata(37:41))/1e4;
            session(i).stdz(k,1) = str2num(fdata(53:57))/1e4;
            session(i).reject(k,1) = cellstr(fdata(58)); %'R' = rejected vectors
        elseif strncmp(C(iCF(k)),'F',1) == 1
            fdata = [];
            fdata = char(C(iCF(f(k))));
            session(i).fromsta(k,1) = str2num(fdata(2:5)); %all from stations in a session (by SSN)
            session(i).tosta(k,1) = str2num(fdata(6:9)); %all to stations in a session        
               
            session(i).dx(k,1) = str2num(fdata(10:22))/1e4; %delta X in meters
            session(i).dy(k,1) = str2num(fdata(28:40))/1e4; %delta Y in meters
            session(i).dz(k,1) = str2num(fdata(46:58))/1e4; %delta Z in meters
            session(i).stdx(k,1) = str2num(fdata(23:27))/1e4; %std X in meters
            session(i).stdy(k,1) = str2num(fdata(41:45))/1e4;
            session(i).stdz(k,1) = str2num(fdata(59:63))/1e4;
            session(i).reject(k,1) = cellstr(fdata(64)); %'R' = rejected vectors
        end
    end
end
%%
%Assign all D-records for each session
for i = 1:length(iB)
    dstart = find(iE > iB(i));
    if i < length(iB)
        dend = find(iE < iB(i+1));
    else
        dend = dstart;
    end
    d = intersect(dstart,dend); %index to all D-records in a session
    for k = 1:length(d)
        ddata = [];
        ddata = char(C(iE(d(k))));
        if length(ddata) == 76
            session(i).drow(k,1:5) = [str2double(ddata(2:4)) str2double(ddata(17:19)) str2double(ddata(32:34)) str2double(ddata(47:49)) str2double(ddata(62:64))];
            session(i).dcol(k,1:5) = [str2double(ddata(5:7)) str2double(ddata(20:22)) str2double(ddata(35:37)) str2double(ddata(50:52)) str2double(ddata(65:67))];
            session(i).corr(k,1:5) = [str2double(ddata(8:16)) str2double(ddata(23:31)) str2double(ddata(38:46)) str2double(ddata(53:61)) str2double(ddata(68:76))]./1e7;
        elseif length(ddata) == 61
            session(i).drow(k,1:5) = [str2double(ddata(2:4)) str2double(ddata(17:19)) str2double(ddata(32:34)) str2double(ddata(47:49)) 0];
            session(i).dcol(k,1:5) = [str2double(ddata(5:7)) str2double(ddata(20:22)) str2double(ddata(35:37)) str2double(ddata(50:52)) 0];
            session(i).corr(k,1:5) = [str2double(ddata(8:16)) str2double(ddata(23:31)) str2double(ddata(38:46)) str2double(ddata(53:61)) 0]./1e7;
        elseif length(ddata) == 46
            session(i).drow(k,1:5) = [str2double(ddata(2:4)) str2double(ddata(17:19)) str2double(ddata(32:34)) 0 0];
            session(i).dcol(k,1:5) = [str2double(ddata(5:7)) str2double(ddata(20:22)) str2double(ddata(35:37)) 0 0];
            session(i).corr(k,1:5) = [str2double(ddata(8:16)) str2double(ddata(23:31)) str2double(ddata(38:46)) 0 0]./1e7;
        elseif length(ddata) == 31
            session(i).drow(k,1:5) = [str2double(ddata(2:4)) str2double(ddata(17:19)) 0 0 0];
            session(i).dcol(k,1:5) = [str2double(ddata(5:7)) str2double(ddata(20:22)) 0 0 0];
            session(i).corr(k,1:5) = [str2double(ddata(8:16)) str2double(ddata(23:31)) 0 0 0]./1e7;
        elseif length(ddata) == 16
            session(i).drow(k,1:5) = [str2double(ddata(2:4)) 0 0 0 0];
            session(i).dcol(k,1:5) = [str2double(ddata(5:7)) 0 0 0 0];
            session(i).corr(k,1:5) = [str2double(ddata(8:16)) 0 0 0 0]./1e7;
        end
    end      
end
%% Upload serfil for converting SSNs to Mark IDs
str = char(SERFIL);
fid = fopen(str);
S = textscan(fid, '%s','Delimiter','');
fclose(fid);
S = S{:};
for i = 1:length(S)
    ids = strsplit(char(S(i)));
    MarkID(i,1) = ids(1);
    SSN(i,1) = str2double(ids(2));
end


%%
% compile G0 lines for each baseline in each session
%starnet input format (with std-correlation matrix)
%G0 'V1 day1a.asc 14.07.2014 09:42:00
%G1 S714-G728   -6824.685850  -17201.203170  -18413.568460
%G2  2.496351000000E-008  4.544639000000E-008  4.928693000000E-008
%G3  2.176306000000E-008 -1.472207000000E-008 -2.624369000000E-008
count = 1;
for i = 1:length(session)
    day = sprintf(' %02d',session(i).day);
    mo = sprintf('%02d',session(i).month);
    yr = sprintf('%4d',session(i).year);
    
    hr = sprintf(' %02d', session(i).hr);
    min = sprintf('%02d', session(i).min);
    
    date = cellstr(strcat(day,'.',mo,'.',yr));
    time = cellstr(strcat(hr,':',min,':00'));
    
    row = 1; %starting row and column for the D-matrix (for first baseline in session)
    col = 2; 
    
    for j = 1:length(session(i).dx)
        session(i).g0(j,:) = strcat('G0 ''V',num2str(count),{' '},session(i).software,date,time);
        count = count+1;
        dxdydz = [' ',num2str(session(i).dx(j)),' ',num2str(session(i).dy(j)),' ',num2str(session(i).dz(j))];
        
        idfrom = find(SSN == session(i).fromsta(j));
        idto = find(SSN == session(i).tosta(j));
        session(i).fromid(j,1) = MarkID(idfrom);
        session(i).toid(j,1) = MarkID(idto);
        
        session(i).g1(j,:) = cellstr(['G1 ',char(session(i).fromid(j)),'-',char(session(i).toid(j)),dxdydz]);
        session(i).g2(j,:) = cellstr(['G2 ',num2str((session(i).stdx(j))),' ',num2str((session(i).stdy(j))),' ',num2str((session(i).stdz(j)))]);
        
        ir1 = find(session(i).drow == row);
        ic2 = find(session(i).dcol == col);
        ic3 = find(session(i).dcol == col+1);
        ir2 = find(session(i).drow == row+1);
        
        i12 = intersect(ir1,ic2);
        i13 = intersect(ir1,ic3);
        i23 = intersect(ir2,ic3);
        
        corr12 = session(i).corr(i12); %correlation for 1,2 of the baseline
        corr13 = session(i).corr(i13);
        corr23 = session(i).corr(i23);
        
        session(i).g3(j,:) = cellstr(['G3 ',num2str(corr12,7),' ',num2str(corr13,7),' ',num2str(corr23,7)]);
        
        row = row+3;
        col = col+3;
    end
    
end
%%
%Output results to a *.gps text file for entry in Starnet
fid = fopen(char(OUTPUT),'w');

fprintf(fid,'%s\r\n\r\n','.GPS WEIGHT STDERRCORR');

for i = 1:length(session)
    for j = 1:length(session(i).dx)
        fprintf(fid,'%s\r\n',char(session(i).g0(j)));
        fprintf(fid,'%s\r\n',char(session(i).g1(j)));
        fprintf(fid,'%s\r\n',char(session(i).g2(j)));
        fprintf(fid,'%s\r\n\r\n',char(session(i).g3(j)));
    end
end
fclose(fid);
        
        
        
        
        
        
        